@@ -34,8 +34,8 @@ from abc import ABCMeta, abstractmethod
 
 import numpy as np
 
-from mslib import netCDF4tools
-from mslib import utils
+from mslib.utils import netCDF4tools
+import mslib.utils.coordinate as coordinate
 
 
 class MSSPlotDriver(metaclass=ABCMeta):
@@ -102,16 +102,15 @@ class MSSPlotDriver(metaclass=ABCMeta):
             self.vert_units = None
             return
 
-        if fc_time < init_time:
-            msg = "Forecast valid time cannot be earlier than " \
-                  "initialisation time."
-            logging.error(msg)
-            raise ValueError(msg)
-        fc_step = fc_time - init_time
-        fc_step = fc_step.days * 24 + (fc_step.seconds // 3600)
+        if self.uses_inittime_dimension():
+            logging.debug("\trequested initialisation time %s", init_time)
+            if fc_time < init_time:
+                msg = "Forecast valid time cannot be earlier than " \
+                      "initialisation time."
+                logging.error(msg)
+                raise ValueError(msg)
         self.fc_time = fc_time
-        logging.debug("\trequested initialisation time %s", init_time)
-        logging.debug("\trequested forecast valid time %s (step %s hrs)", fc_time, fc_step)
+        logging.debug("\trequested forecast valid time %s", fc_time)
 
         # Check if a dataset is open and if it contains the requested times.
         # (a dataset will only be open if the used layer has not changed,
@@ -421,10 +420,11 @@ class VerticalSectionDriver(MSSPlotDriver):
         """
         logging.debug("computing %i interpolation points, connection: %s",
                       vsec_numpoints, vsec_path_connection)
-        now = datetime.now()
-        self.lats, self.lons, _ = utils.path_points(
-            [(_x, _y, now) for _x, _y in vsec_path],
+        self.lats, self.lons = coordinate.path_points(
+            [_x[0] for _x in vsec_path],
+            [_x[1] for _x in vsec_path],
             numpoints=vsec_numpoints, connection=vsec_path_connection)
+        self.lats, self.lons = np.asarray(self.lats), np.asarray(self.lons)
         self.vsec_path = vsec_path
         self.vsec_numpoints = vsec_numpoints
         self.vsec_path_connection = vsec_path_connection
@@ -479,8 +479,7 @@ class VerticalSectionDriver(MSSPlotDriver):
             logging.debug("\tInterpolating to cross-section path.")
             # Re-arange longitude dimension in the data field.
             var_data = var_data[:, :, lon_indices]
-            data[name] = utils.interpolate_vertsec(var_data, self.lat_data, lon_data,
-                                                   self.lats, self.lons)
+            data[name] = coordinate.interpolate_vertsec(var_data, self.lat_data, lon_data, self.lats, self.lons)
             # Free memory.
             del var_data
 
@@ -725,10 +724,13 @@ class LinearSectionDriver(VerticalSectionDriver):
         """
         logging.debug("computing %i interpolation points, connection: %s",
                       lsec_numpoints, lsec_path_connection)
-        now = datetime.now()
-        self.lats, self.lons, self.alts, _ = utils.path_points(
-            [(_x, _y, _z, now) for _x, _y, _z in lsec_path],
-            numpoints=lsec_numpoints, connection=lsec_path_connection, contains_altitude=True)
+        self.lats, self.lons, self.alts = coordinate.path_points(
+            [_x[0] for _x in lsec_path],
+            [_x[1] for _x in lsec_path],
+            alts=[_x[2] for _x in lsec_path],
+            numpoints=lsec_numpoints, connection=lsec_path_connection)
+        self.lats, self.lons, self.alts = [
+            np.asarray(_x) for _x in (self.lats, self.lons, self.alts)]
         self.lsec_path = lsec_path
         self.lsec_numpoints = lsec_numpoints
         self.lsec_path_connection = lsec_path_connection
@@ -793,7 +795,7 @@ class LinearSectionDriver(VerticalSectionDriver):
             # Re-arange longitude dimension in the data field.
             var_data = var_data[:, :, lon_indices]
 
-            cross_section = utils.interpolate_vertsec(var_data, self.lat_data, lon_data, self.lats, self.lons)
+            cross_section = coordinate.interpolate_vertsec(var_data, self.lat_data, lon_data, self.lats, self.lons)
             # Create vertical interpolation factors and indices for subsequent variables
             # TODO: Improve performance for this interpolation in general
             if len(factors) == 0:
